﻿using System;
using System.Drawing;
using System.Windows.Forms;
using OpenTK;
using OpenTK.Graphics;
using OpenTK.Graphics.OpenGL;

namespace Sin
{
    public partial class Form1 : Form
    {
        private GLControl _glControl;
        private const float _fieldWidth = 4f * (float)Math.PI;
        private const float _fieldHeight = 4f;
        private const float _halfFieldWidth = _fieldWidth / 2f;
        private const float _halfFieldHeight = _fieldHeight / 2f;
        private bool _isAxesShown;

        public Form1()
        {
            InitializeComponent();

            _glControl = new GLControl();
            _glControl.Load += GLControl_Load;
            _glControl.Paint += GLControl_Paint;
            _glControl.Dock = DockStyle.Fill;
            _tableLayoutPanel1.Controls.Add(_glControl, 0, 0);
            _tableLayoutPanel1.SetColumnSpan(_glControl, 2);
            _isAxesShown = _checkBoxShowAxes.Checked;
        }

        private void GLControl_Load(object sender, EventArgs e)
        {
            GL.ClearColor(new Color4(0.894f, 0.945f, 0.941f, 1f));
            Setup2DGraphics(_fieldWidth, _fieldHeight);
        }

        private void Setup2DGraphics(float width, float height)
        {
            float halfWidth = width / 2f;
            float halfHeight = height / 2f;
            GL.MatrixMode(MatrixMode.Projection);
            GL.LoadIdentity();
            GL.Ortho(-halfWidth, halfWidth, -halfHeight, halfHeight, 100, -100);
            GL.MatrixMode(MatrixMode.Modelview);
            GL.LoadIdentity();
        }

        private void GLControl_Paint(object sender, PaintEventArgs e)
        {
            GL.Clear(ClearBufferMask.ColorBufferBit);

            DrawAxes();
            DrawSin(Color4.DarkBlue);

            _glControl.SwapBuffers();
        }

        private void DrawLine(PointF startPoint, PointF endPoint, Color4 color)
        {
            GL.Color4(color);
            GL.Begin(PrimitiveType.Lines);
            {
                GL.Vertex2(startPoint.X, startPoint.Y);
                GL.Vertex2(endPoint.X, endPoint.Y);
            }
            GL.End();
        }

        private void DrawAxes()
        {
            if (!_isAxesShown)
            {
                return;
            }

            // X
            DrawLine(new PointF(-_halfFieldWidth, 0f), new PointF(_halfFieldWidth, 0f), Color4.Red);

            // Y
            DrawLine(new PointF(0f, -_halfFieldHeight), new PointF(0f, _halfFieldHeight), Color4.Green);
        }

        private void DrawSin(Color4 color)
        {
            const float step = (float)Math.PI / 20f;

            float xPrev = -2 * (float)Math.PI;

            for (int i = 0; i < 80; i++)
            {
                float x0 = xPrev;
                float y0 = (float)Math.Sin(x0);

                float x1 = x0 + step;
                float y1 = (float)Math.Sin(x1);
                xPrev = x1;

                DrawLine(new PointF(x0, y0), new PointF(x1, y1), color);
            }
        }

        private void BtnShowBGColorDialog_Click(object sender, EventArgs e)
        {
            using (var colorDialog = new ColorDialog())
            {
                if (colorDialog.ShowDialog() == DialogResult.OK)
                {
                    GL.ClearColor(colorDialog.Color);
                    _glControl.Invalidate();
                }
            }
        }

        private void CheckBoxShowAxes_CheckedChanged(object sender, EventArgs e)
        {
            _isAxesShown = !_isAxesShown;
            _glControl.Invalidate();
        }
    }
}
